package com.kob.botrunningsystem.service.impl.utils;

import java.util.LinkedList;
import java.util.Queue;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;

public class BotPool extends Thread {
    // 定义可重入锁
    private final ReentrantLock lock = new ReentrantLock();
    // 定义条件变量:如果队列空,线程会阻塞;一旦有新的任务输入,则唤醒线程
    private final Condition condition = lock.newCondition();
    // 定义队列: Queue 会在两个线程中操作 ==> 生产者会生产任务 ; 消费者会消费任务
    private final Queue<Bot> bots = new LinkedList<>();

    // 往队列中插入一个 bot
    public void addBot(Integer userId, String botCode, String input) {
        // 这里涉及对 Queue 的操作,需要枷锁
        lock.lock();
        try {
            bots.add(new Bot(userId, botCode, input));  // 给队列添加新的 bot
            condition.signalAll();  // 唤醒另外一个线程
        } finally {
            lock.unlock();
        }
    }

    // 消费任务:编译执行代码(取出来的任务) ==> 使用 JOOR
    // 每次执行一个代码都开一个线程,用来操控执行时间
    private void consume(Bot bot) {
        // 将代码发送给 Consumer
        Consumer consumer = new Consumer(); // 定义一个 consumer
        consumer.startTimeout(2000,bot);    // 每个 bot 每回合最多执行 2 秒(每个玩家每回合设定等待 5 秒输入)
    }


    // 如果队列空,线程会阻塞;一旦有新的任务输入,则唤醒线程
    @Override
    public void run() {
        // 循环消费任务
        while (true) {
            lock.lock();
            // 如果 bots 队列为空,则阻塞当前线程
            if (bots.isEmpty()) {
                try {
                    condition.await();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                    lock.unlock();  // 报异常要解锁
                    break;  // 报异常则直接 break 中断进程
                }
            }

            // 如果 bots 不空,则
            else {
                // 取出队头任务
                Bot bot = bots.remove();    // Queue.remove() 返回并删除对头元素
                lock.unlock();  // 取完队头,即解锁

                // 消费取出来的任务: consume() 比较耗时,因此需要在执行之前先解锁 lock.unlock
                consume(bot);
            }
        }
    }
}
